package spdwithexpectationonbanetwork;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.graph.Network;
import repast.simphony.util.ContextUtils;

public class Player {
	private int ID;
	private int expectationCurrent;
	private int expectationPrevious;
	private double initia;
	private char currentStrategy;
	private char choosedStrategy;
	private double currentSocialPreferencePayoff;
	private double theta;  //EWA coefficient
	private double alpha;
	private double beta;
	private double lamda;
	private double payoffOfCC;
	private double payoffOfCD;
	private double payoffOfDC;
	private double payoffOfDD;
	private boolean isIfSocialPreference;
	private double currentPayoffCooperation=0;
	private double currentPayoffDefection=0;
	private double currentPayoff;
	private double  totalPayoff=0;
	private double  realPayoff=0;
	private int isCooperate=0;
	double defectPayoff;
	double defectSocialPayoff;
	int realCooperation;
	double expectationPayoffCooperation;
	double expectationPayoffDefection;
	double expectationInitial;
	boolean hasExpectation;
	
	public Player( ){
		//initialize  agent  parameter
		Parameters p = RunEnvironment.getInstance().getParameters();
        this.payoffOfCC=(Double)p.getValue("payoffOfCC");
		this.payoffOfCD=(Double)p.getValue("payoffOfCD");
		this.payoffOfDC=(Double)p.getValue("payoffOfDC");
		this.payoffOfDD=(Double)p.getValue("payoffOfDD");
		this.lamda=(Double)p.getValue("lamda");
		this.initia=(Double)p.getValue("initia");
		this.currentStrategy='N';
		this.hasExpectation=false;
	}
	
	public void setInitialExpectation(){
    	Controller context = (Controller) ContextUtils.getContext(this);
        Network network=(Network)context.getProjection("Living world");
    	int degree=network.getDegree(this);
    	this.expectationPrevious=(int)Math.round(degree*expectationInitial);
    	this.expectationCurrent=this.expectationPrevious;
    	//System.out.println(" expectation current  "+expectationCurrent);
    	//System.out.println("degree of this agent    "+degree);
	}
		
	public double singlePlayCooperation(Player agent1,Player agent2){
		if (agent2.currentStrategy=='D'){
			//System.out.println("payoff of CD  is  "+payoffOfCD);
			return payoffOfCD;
		}
		else 	{
			//System.out.println("payoff of CC  is  "+payoffOfCC);
			return payoffOfCC;
		   }
		}
	
	public double singlePlayDefection(Player agent1,Player agent2){
		if (agent2.currentStrategy=='D'){
		//System.out.println("payoff of DD  is  "+payoffOfDD);		
		return payoffOfDD;
		       }
		else{ 	
			//System.out.println("payoff of DC  is  "+payoffOfDC);		
			return payoffOfDC;
		    }
		}
	
    public double computePayoffCooperation() {
	    //compute the payoff and the socialPreference if needed in initial status
        currentPayoffCooperation=0;

        Controller context = (Controller) ContextUtils.getContext(this);
        Network network=(Network)context.getProjection("Living world");
	    Iterable neighbors=network.getAdjacent(this);
	    //System.out.println("begin compute the payoff  ");
	    
	    for (Object o : neighbors) {
	    	currentPayoffCooperation+=(double)singlePlayCooperation(this,(Player)o);
	       			//System.out.println("pay off is computing   added   "+((Player)o).getID()+"  "+singlePlayCooperation(this,(Player)o));
	    }  
	    //System.out.println(" payoff computed  for  cooperation  of agent  "+this.ID+"    "+currentPayoffCooperation);
	     return currentPayoffCooperation;
	}
    
    public double computePayoffDefection() {
	    //compute the payoff and the socialPreference if needed in initial status
        currentPayoffDefection=0;

        Controller context = (Controller) ContextUtils.getContext(this);
        Network network=(Network)context.getProjection("Living world");
	    Iterable neighbors=network.getAdjacent(this);
	    //System.out.println("begin compute the payoff  ");
	    
	    for (Object o : neighbors) {
	    	currentPayoffDefection+=(double)singlePlayDefection(this,(Player)o);
	       			//System.out.println("pay off is computing");
	    }  
	     //System.out.println(" payoff computed  for  Defection   of agent "+this.ID+"    "+currentPayoffDefection);
	     return currentPayoffDefection;
	}
    
    public void computeCurrentPayoff(){
    	if(this.getCurrentStrategy()=='C'){
    		currentPayoff=computePayoffCooperation() ;
    		//System.out.println("the payoff is (C)  "+currentPayoff);
    	}
    	else if (this.getCurrentStrategy()=='D'){
    		currentPayoff=computePayoffDefection() ;
    		//System.out.println("the payoff is (D)  "+currentPayoff);
    	}
    }
    
   public void expectationAdjust(){
	   Controller context = (Controller) ContextUtils.getContext(this);
       Network network=(Network)context.getProjection("Living world");
	    Iterable neighbors=network.getAdjacent(this);
	    realCooperation=0;
	 	int size=0;
	 	
	    for (Object o : neighbors) {
	    	if (((Player)o).getCurrentStrategy()=='C')realCooperation++;
	    	size++;
	    }  
	    //System.out.println("real and previous expectation of cooperator  "+realCooperation+"   "+expectationPrevious);
	    if(!isHasExpectation())initia=0;
	    //System.out.println(" has expectation   "+isHasExpectation());
	    expectationCurrent=(int)(expectationPrevious+Math.round((realCooperation-expectationPrevious)*(1-initia)));
	    //System.out.println("Current and previous expectation   "+expectationCurrent+"    "+
	    //expectationPrevious+" total degree    "+size+"      realcooperator     "+realCooperation);
    }
    
    public void computeExpectationPayoff(){
    	Controller context = (Controller) ContextUtils.getContext(this);
        Network network=(Network)context.getProjection("Living world");
    	int degree=network.getDegree(this);
    	int expectationDefector=degree-expectationCurrent;
    	//System.out.println("expectation of defector  "+expectationDefector+"       degree     "+degree);
    	expectationPayoffCooperation=expectationCurrent*payoffOfCC+expectationDefector*payoffOfCD;
    	expectationPayoffDefection=expectationDefector*payoffOfDD+expectationCurrent*payoffOfDC;
    }
    
    public void chooseStrategy(){
    	double prob=0;
    	//RandomHelper.init();
    	RandomHelper.createUniform();
    	double randomProb=RandomHelper.nextDoubleFromTo(0, 1);
    	//System.out.println("total payoff   expectation payoff C  expectation D "+this.getCurrentPayoff()
    	//+"  "+expectationPayoffCooperation+"    "+expectationPayoffDefection+"   random generated   "+randomProb);
    	
    	Controller context = (Controller) ContextUtils.getContext(this);
        Network network=(Network)context.getProjection("Living world");
    	int degree=network.getDegree(this);
    	int expectationDefector=degree-expectationCurrent;
    	
    	if(this.getCurrentStrategy()=='D'){
    		prob=(double)1/(1+Math.exp(lamda*(this.getCurrentPayoff()-expectationPayoffCooperation)));
    		//System.out.println("prob of C   "+prob+"  "+this.ID);
    	}
    	else {
    		prob=(double)1/(1+Math.exp(lamda*(this.getCurrentPayoff()-expectationPayoffDefection)));
    		//System.out.println("prob of D   "+prob+"   "+this.ID);
    	}
    	
    	  if (randomProb<=prob) {
  	    	//System.out.println("    random strategy  is  "+random.getCurrentStrategy());
  	    	 if (this.getCurrentStrategy()=='D'){
  	    		  this.choosedStrategy='C';
//  	    		  System.out.println("choose C  "+this.ID+"   BECAUSE   "
//  	    				  + "   real payoff  "+this.getCurrentPayoff()+
//  	    				  "      expectation payoff   of cooperation  "+expectationPayoffCooperation
//  	    				  +"    real cooperator   "+realCooperation+ " previous   "+expectationPrevious
//  	    				  +"   current expectation of cooperation   "+expectationCurrent);
  	    		     
  	    	 }
  	    	 else {
  	    		 this.choosedStrategy='D';
//  	    		  System.out.println("choose D  "+this.ID+" BECAUSE   "+expectationCurrent
//  	    				  + "   real payoff  "+this.getCurrentPayoff()+
//  	    				  "      expectation payoff  of defection   "+expectationPayoffDefection
//  	    				  +"    real cooperator   "+realCooperation+ " previous   "+expectationPrevious
//  	    				  +"   current exoectation of  defection   "+expectationDefector);
  	    	   }
  	    	 }
  	      else{
  	    	this.choosedStrategy=this.getCurrentStrategy();
  	    	//System.out.println("strategy no changed");
  	      };
  	    
       	if(this.choosedStrategy=='C'){
       		isCooperate=1;
       		//System.out.println("choosed Cooperation");
       	}
       	else {
       		isCooperate=0;
       		//System.out.println("choose defection");
       	}
    }
    
    public void step1() {
    	computeCurrentPayoff();
    	}
    
    public void step2() {
    	expectationAdjust();
    	computeExpectationPayoff();
    }
     
    public void step3(){
    	chooseStrategy();
    }
    
    public void postStep(){
        this.currentStrategy=choosedStrategy;
        this.expectationPrevious=expectationCurrent;
    }
    
   	/**
	 * @return the lamda
	 */
	public double getLamda() {
		return lamda;
	}

	/**
	 * @param lamda the lamda to set
	 */
	public void setLamda(double lamda) {
		this.lamda = lamda;
	}

	/**
	 * @return the realPayoff
	 */
	public double getRealPayoff() {
		return realPayoff;
	}

	/**
	 * @param realPayoff the realPayoff to set
	 */
	public void setRealPayoff(double realPayoff) {
		this.realPayoff = realPayoff;
	}

	/**
	 * @return the alpha
	 */
	public double getAlpha() {
		return alpha;
	}

	/**
	 * @param alpha the alpha to set
	 */
	public void setAlpha(double alpha) {
		this.alpha = alpha;
	}

	/**
	 * @return the beta
	 */
	public double getBeta() {
		return beta;
	}

	/**
	 * @param beta the beta to set
	 */
	public void setBeta(double beta) {
		this.beta = beta;
	}

	/**
	 * @return the isIfSocialPreference
	 */
	public boolean isIfSocialPreference() {
		return isIfSocialPreference;
	}

	/**
	 * @param isIfSocialPreference the isIfSocialPreference to set
	 */
	public void setIfSocialPreference(boolean isIfSocialPreference) {
		this.isIfSocialPreference = isIfSocialPreference;
	}

	/**
	 * @return the currentPayoffCooperation
	 */
	public double getCurrentPayoffCooperation() {
		return currentPayoffCooperation;
	}

	/**
	 * @param currentPayoffCooperation the currentPayoffCooperation to set
	 */
	public void setCurrentPayoffCooperation(double currentPayoffCooperation) {
		this.currentPayoffCooperation = currentPayoffCooperation;
	}

	/**
	 * @return the currentPayoffDefection
	 */
	public double getCurrentPayoffDefection() {
		return currentPayoffDefection;
	}

	/**
	 * @param currentPayoffDefection the currentPayoffDefection to set
	 */
	public void setCurrentPayoffDefection(double currentPayoffDefection) {
		this.currentPayoffDefection = currentPayoffDefection;
	}

	/**
	 * @return the isIfSocialPrefer
	 */
	public boolean isIfSocialPrefer() {
		return isIfSocialPreference;
	}

	/**
	 * @param isIfSocialPrefer the isIfSocialPrefer to set
	 */
	public void setIfSocialPrefer(boolean isIfSocialPreference) {
		this.isIfSocialPreference = isIfSocialPreference;
	}

	/**
	 * @return the currentStrategy
	 */
	public char getCurrentStrategy() {
		return currentStrategy;
	}

	/**
	 * @param currentStrategy the currentStrategy to set
	 */
	public void setCurrentStrategy(char currentStrategy) {
		this.currentStrategy = currentStrategy;
	}

	public char getChoosedStrategy() {
		return choosedStrategy;
	}

	/**
	 * @param choosedStrategy the choosedStrategy to set
	 */
	public void setChoosedStrategy(char choosedStrategy) {
		this.choosedStrategy = choosedStrategy;
	}

	public double getTheta() {
		return theta;
	}

	/**
	 * @param theta the theta to set
	 */
	public void setTheta(double theta) {
		this.theta = theta;
	}

	/**
	 * @return the previousN
	 */

	/**
	 * @return the currentN
	 */

	/**postStep()
	 * @return the payoffOfCC
	 */
	public double getPayoffOfCC() {
		return payoffOfCC;
	}

	/**
	 * @param payoffOfCC the payoffOfCC to set
	 */
	public void setPayoffOfCC(double payoffOfCC) {
		this.payoffOfCC = payoffOfCC;
	}

	/**
	 * @return the payoffOfCD
	 */
	public double getPayoffOfCD() {
		return payoffOfCD;
	}

	/**
	 * @param payoffOfCD the payoffOfCD to set
	 */
	public void setPayoffOfCD(double payoffOfCD) {
		this.payoffOfCD = payoffOfCD;
	}

	/**
	 * @return the payoffOfDC
	 */
	public double getPayoffOfDC() {
		return payoffOfDC;
	}

	/**
	 * @param payoffOfDC the payoffOfDC to set
	 */
	public void setPayoffOfDC(double payoffOfDC) {
		this.payoffOfDC = payoffOfDC;
	}

	/**
	 * @return the payoffOfDD
	 */
	public double getPayoffOfDD() {
		return payoffOfDD;
	}

	/**
	 * @param payoffOfDD the payoffOfDD to set
	 */
	public void setPayoffOfDD(double payoffOfDD) {
		this.payoffOfDD = payoffOfDD;
	}

	/**
	 * @return the iD
	 */
	public int getID() {
		return ID;
	}

	/**
	 * @return the totalPayoff
	 */
	public double getTotalPayoff() {
		return totalPayoff;
	}

	/**
	 * @param totalPayoff the totalPayoff to set
	 */
	public void setTotalPayoff(double totalPayoff) {
		this.totalPayoff = totalPayoff;
	}

	/**
	 * @param id the iD to set
	 */
	public void setID(int id) {
		ID = id;
	}

	/**
	 * @return the isCooperate
	 */
	public int getIsCooperate() {
		return isCooperate;
	}

	/**
	 * @param isCooperate the isCooperate to set
	 */
	public void setIsCooperate(int isCooperate) {
		this.isCooperate = isCooperate;
	}

	/**
	 * @return the currentPayoff
	 */
	public double getCurrentPayoff() {
		return currentPayoff;
	}

	/**
	 * @return the currentSocialPreferencePayoff
	 */
	public double getCurrentSocialPreferencePayoff() {
		return currentSocialPreferencePayoff;
	}

	/**
	 * @param currentSocialPreferencePayoff the currentSocialPreferencePayoff to set
	 */
	public void setCurrentSocialPreferencePayoff(
			double currentSocialPreferencePayoff) {
		this.currentSocialPreferencePayoff = currentSocialPreferencePayoff;
	}

	/**
	 * @param currentPayoff the currentPayoff to set
	 */
	public void setCurrentPayoff(double currentPayoff) {
		this.currentPayoff = currentPayoff;
	}

	/**
	 * @return the expectationCurrent
	 */
	public int getExpectationCurrent() {
		return expectationCurrent;
	}

	/**
	 * @param expectationCurrent the expectationCurrent to set
	 */
	public void setExpectationCurrent(int expectationCurrent) {
		this.expectationCurrent = expectationCurrent;
	}

	/**
	 * @return the expectationPrevious
	 */
	public int getExpectationPrevious() {
		return expectationPrevious;
	}

	/**
	 * @param expectationPrevious the expectationPrevious to set
	 */
	public void setExpectationPrevious(int expectationPrevious) {
		this.expectationPrevious = expectationPrevious;
	}

	/**
	 * @return the initia
	 */
	public double getInitia() {
		return initia;
	}

	/**
	 * @param initia the initia to set
	 */
	public void setInitia(double initia) {
		this.initia = initia;
	}

	/**
	 * @return the defectPayoff
	 */
	public double getDefectPayoff() {
		return defectPayoff;
	}

	/**
	 * @param defectPayoff the defectPayoff to set
	 */
	public void setDefectPayoff(double defectPayoff) {
		this.defectPayoff = defectPayoff;
	}

	/**
	 * @return the defectSocialPayoff
	 */
	public double getDefectSocialPayoff() {
		return defectSocialPayoff;
	}

	/**
	 * @param defectSocialPayoff the defectSocialPayoff to set
	 */
	public void setDefectSocialPayoff(double defectSocialPayoff) {
		this.defectSocialPayoff = defectSocialPayoff;
	}

	
	
	/**
	 * @return the hasExpectation
	 */
	public boolean isHasExpectation() {
		return hasExpectation;
	}

	/**
	 * @param hasExpectation the hasExpectation to set
	 */
	public void setHasExpectation(boolean hasExpectation) {
		this.hasExpectation = hasExpectation;
	}

	/**
	 * @return the realCooperation
	 */
	public int getRealCooperation() {
		return realCooperation;
	}

	/**
	 * @param realCooperation the realCooperation to set
	 */
	public void setRealCooperation(int realCooperation) {
		this.realCooperation = realCooperation;
	}

	/**
	 * @return the expectationPayoffCooperation
	 */
	public double getExpectationPayoffCooperation() {
		return expectationPayoffCooperation;
	}

	/**
	 * @param expectationPayoffCooperation the expectationPayoffCooperation to set
	 */
	public void setExpectationPayoffCooperation(double expectationPayoffCooperation) {
		this.expectationPayoffCooperation = expectationPayoffCooperation;
	}

	/**
	 * @return the expectationPayoffDefection
	 */
	public double getExpectationPayoffDefection() {
		return expectationPayoffDefection;
	}

	/**
	 * @param expectationPayoffDefection the expectationPayoffDefection to set
	 */
	public void setExpectationPayoffDefection(double expectationPayoffDefection) {
		this.expectationPayoffDefection = expectationPayoffDefection;
	}

	/**
	 * @return the expectationInitial
	 */
	public double getExpectationInitial() {
		return expectationInitial;
	}

	/**
	 * @param expectationInitial the expectationInitial to set
	 */
	public void setExpectationInitial(double expectationInitial) {
		this.expectationInitial = expectationInitial;
	}
}